<?php

function build_category_table(){
    $settings = get_option('rapidline_settings');

    if(!$settings){
        $settings = array();
    }

    $table = [];

    $table[] = '<table border="1" cellpadding="5" cellspacing="0" width="100%">';

    $table[] = '<tr>';
    $table[] = '<th width="20"><input type="checkbox" class="select-all" data-select_selector=".categories_checkbox"></th>';
    $table[] = '<th>Category</th>';
    $table[] = '<th>Markup</th>';
    $table[] = '<th>Category</th>';
    $table[] = '</tr>';

    $rows = build_category_table_rows($settings);

    $table = [...$table, ...$rows];

    $table[] = '</table>';

    return implode("\n", $table);
}

function build_category_table_rows($settings = [], $parent_id = null, $name_prefix = ''){
    $rows = [];

    $rapidline = new \Rapidline\API(get_option('rl_api_key'));

    $categories = $rapidline->categories($parent_id);

    if($categories and count($categories)){
        foreach($categories as $category){
            $category_name = ($name_prefix?$name_prefix.' > ':'').$category['name'];

            $category_id = -1;
            $markup = 100;
            $checked = '';

            if(isset($settings[$category['id']])){
                if(!empty($settings[$category['id']]['markup'])){
                    $markup = $settings[$category['id']]['markup'];
                }
                
                if(!empty($settings[$category['id']]['import']) and $settings[$category['id']]['import']){
                    $checked = ' checked';
                }

                $category_id = rapidline_object('category', $category['id']);
            }

            $rows[] = '<tr>';
            $rows[] = '<td><input type="checkbox" class="categories_checkbox" name="settings['.$category['id'].'][import]" value="1"'.$checked.'></td>';
            $rows[] = '<td>'.$category_name.'</td>';
            $rows[] = '<td><input type="number" name="settings['.$category['id'].'][markup]" value="'.number_format($markup, 2).'" step="0.01"> %</td>';
            $rows[] = '<td>'.category_selectbox('settings['.$category['id'].'][category_id]', $category_id).'</td>';
            $rows[] = '</tr>';

            $child_rows = build_category_table_rows($settings, $category['id'], $category_name);

            $rows = [...$rows, ...$child_rows];
        }
    }

    return $rows;
}

function category_selectbox($name, $value){
    $output = [];

    $output[] = '<select name="'.$name.'">';

    $output[] = '<option value="-1">Create New Category</option>';

    $output[] = '<optgroup label="Existing Categories">';

    $category_options = category_selectbox_options(0, '', $value);

    $output = [...$output, ...$category_options];

    $output[] = '</optgroup>';

    $output[] = '</select>';

    return implode("\n", $output);
}

function category_selectbox_options($parent_id = 0, $parent_name = '', $value = -1){
    $output = [];

    $product_categories = get_terms( 'product_cat', [
        'parent' => $parent_id,
        'orderby' => 'name',
        'order' => 'ASC',
        'hide_empty' => false,
    ]);

    foreach ($product_categories as $key => $category) {
        $selected_text = ($value == $category->term_id?' selected':'');

        $category_name = ($parent_name?$parent_name.' > ':'').$category->name;

        $output[] = '<option value="'.$category->term_id.'"'.$selected_text.'>'.$category_name.'</option>';

        $category_options = category_selectbox_options($category->term_id, $category_name, $value);

        if(count($category_options)){
            $output = [...$output, ...$category_options];
        }
    }

    return $output;
}

function rapidline_object($type, $id){
    global $wpdb;

    if($type == 'product' or $type == 'attachment'){
        $results = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}postmeta` WHERE `meta_key` = '_rapidline_id' AND `meta_value` = '$id'");

        if($results and isset($results[0]) and $results[0]){
            return $results[0]->post_id;
        }
    }else{
        if($type == 'category'){
            $settings = get_option('rapidline_settings');

            if(!$settings){
                $settings = array();
            }

            if(isset($settings[$id])){
                $category_id = $settings[$id]['category_id'];

                if($category_id != -1){
					if(get_term($category_id, 'product_cat')){
                    	return $category_id;
					}
                }
            }
        }
    }
    
    return false;
}

function wp_attachment_insert_path($file_path, $post_id, $filename = false){
    global $wpdb;

    require_once(ABSPATH . 'wp-admin/includes/image.php');
    require_once(ABSPATH . 'wp-admin/includes/file.php');
    require_once(ABSPATH . 'wp-admin/includes/media.php');

    if($filename == false){
        $filename = basename($file_path);
    }

    $upload_file = wp_upload_bits($filename, null, file_get_contents($file_path));

    if($upload_file['error']){
        return false;
    }

    $wp_filetype = wp_check_filetype($filename, null);

    $attachment = array(
        'post_mime_type' => $wp_filetype['type'],
        'post_parent' => $post_id,
        'post_title' => preg_replace('/\.[^.]+$/', '', $filename),
        'post_content' => '',
        'post_status' => 'inherit'
    );

    $attachment_id = wp_insert_attachment($attachment, $upload_file['file'], $post_id);
    
    if(is_wp_error($attachment_id)){
        return false;
    }

    $attachment_data = wp_generate_attachment_metadata($attachment_id, $upload_file['file']);
    
    wp_update_attachment_metadata($attachment_id, $attachment_data);

    return $attachment_id;
}

function file_download($url, $file_path){
    $fp = fopen ($file_path, 'w+');              // open file handle

    if($fp){
        $ch = curl_init($url);
        // curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // enable if you want
        curl_setopt($ch, CURLOPT_FILE, $fp);          // output to file
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 1000);      // some large value to allow curl to run for a long time
        curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0');
        
        if(curl_exec($ch)){
            curl_close($ch);
            fclose($fp);

            return true;
        }

        fclose($fp);
    }
    return false;
}

function wc_get_product_object_type( $type ) {
    // Get an instance of the WC_Product object (depending on his type)
    if(isset($type) && $type === 'variable'){
        $product = new WC_Product_Variable();
    }elseif(isset($type) && $type === 'grouped'){
        $product = new WC_Product_Grouped();
    }elseif(isset($type) && $type === 'external'){
        $product = new WC_Product_External();
    }elseif(isset($type) && $type === 'variation'){
        $product = new WC_Product_Variation();
    }else{
        $product = new WC_Product_Simple(); // "simple" By default
    }
    
    if( ! is_a( $product, 'WC_Product' ) )
        return false;
    else
        return $product;
}