<?php
namespace RL\Admin;

class Import {

	public function __construct(){
		add_action('admin_menu', array($this, 'admin_menu'));
		
		add_action('wp_ajax_rl_download_log', array($this, 'wp_ajax_rl_download_log'));
		add_action('wp_ajax_rl_import', array($this, 'wp_ajax_rl_import'));
		add_action('wp_ajax_rl_import_process', array($this, 'wp_ajax_rl_import_process'));
		add_action('wp_ajax_rl_import_reset', array($this, 'wp_ajax_rl_import_reset'));
		add_action('wp_ajax_rl_import_settings', array($this, 'wp_ajax_rl_import_settings'));
		add_action('wp_ajax_rl_set_api_key', array($this, 'wp_ajax_rl_set_api_key'));

		add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_scripts'));
	}

	public function admin_enqueue_scripts(){
		wp_enqueue_script('rl_import_script', RL_IMPORTER_BASE_URL.'/admin/js/scripts.js', array('jquery'), RL_IMPORTER_VERSION, true);
	}
	
	public function admin_menu(){
		add_management_page('Rapidline Import', 'Rapidline Import', 'administrator', 'rl-admin-import', array($this, 'admin_page'));
	}

	static function get_import_dir(){
		$upload_dir = wp_upload_dir();
		$import_dir = $upload_dir['basedir'].'/rapidline-import';

		if(file_exists($import_dir)){
			if(is_dir($import_dir)){
				@mkdir($import_dir.'/data', 0777, true);
				return $import_dir;
			}
		}

		@mkdir($import_dir, 0777, true);
		@mkdir($import_dir.'/data', 0777, true);

		return $import_dir;
	}

	public function admin_page(){
		$task = (empty($_GET['task'])?false:$_GET['task']);

		switch($task){
			case 'errors':
				include(RL_IMPORTER_BASE_PATH.'/templates/errors.php');
			break;
			default:
				if(FALSE === get_option('rl_api_key') && FALSE === update_option('rl_api_key',FALSE)){
					//add_option('my_option',$default_value);
					include(RL_IMPORTER_BASE_PATH.'/templates/api_key.php');
				}else{
					$task_manager = new \RL\TaskManager(self::get_import_dir());
		
					$running = $task_manager->running();
					
					if($task_manager->active()){
						\RL\Cron::init();
						include(RL_IMPORTER_BASE_PATH.'/templates/progress.php');
					}else{
						include(RL_IMPORTER_BASE_PATH.'/templates/import.php');
					}
				}
			break;
		}
	}

	public function wp_ajax_rl_set_api_key(){
		if(!empty($_POST['api_key'])){
			$api_key = $_POST['api_key'];

			$rapidline = new \Rapidline\API($api_key);

			if(false !== $rapidline->ping()){
				add_option('rl_api_key', $api_key);
			}
		}
		
		header('location: '.admin_url('tools.php?page=rl-admin-import'));
		exit();
	}

	public function wp_ajax_rl_import_reset(){
		$task_manager = new \RL\TaskManager(self::get_import_dir());

		$task_manager->cancel();

		header('location: '.admin_url('tools.php?page=rl-admin-import'));
		exit();
	}

	public function wp_ajax_rl_import(){
		$task_manager = \RL\TaskManager::create(self::get_import_dir());

		$settings = get_option('rapidline_settings');

		if(!$settings){
			$settings = array();
		}

		foreach($settings as $category_id => $data){
			if(!empty($data['import']) and $data['import']){
				$task_manager->add_todo('add_category', [
					'category_id' => $category_id
				]);
			}
		}

		\RL\Cron::init();
		
		header('location: '.admin_url('tools.php?page=rl-admin-import'));
		exit();
	}

	public function wp_ajax_rl_import_settings(){
		update_option('rapidline_settings', $_POST['settings']);
		header('location: '.admin_url('tools.php?page=rl-admin-import'));
		exit();
	}

	public function wp_ajax_rl_import_process(){
		$this->return_status();
	}

	public function return_status(){
		$task_manager = new \RL\TaskManager(self::get_import_dir());

		$messages = $task_manager->view_log();

		$messages = array_reverse(explode("\n", $messages));

		$messages = implode("\n", $messages);

		header('content-type: application/json');
		print json_encode(array(
			'current_time' => time(),
			'start_time' => $task_manager->start_time(),
			'completed' => $task_manager->tasks_completed(),
			'total' => $task_manager->tasks_total(),
			'messages' => $messages
		));
		exit();
	}

}