<?php
namespace RL;

class Cron {

	public $running_cron = false;

	public function __construct(){
		add_filter('cron_schedules', array($this, 'cron_schedules'));
		add_action('rl_cron_import_process', array($this, 'import_process'));
		add_filter('wp_kses_allowed_html', array($this, 'wp_kses_allowed_html'), 10, 2);
	}

	public static function init(){
		$timestamp = wp_next_scheduled('rl_cron_import_process');
		wp_unschedule_event($timestamp, 'rl_cron_import_process');

		if(!wp_next_scheduled('rl_cron_import_process')){
			wp_schedule_single_event(time()+2, 'rl_cron_import_process');
		}
	}

	public function wp_kses_allowed_html($tags, $context){
		if($this->running_cron){
			$tags['iframe'] = array(
				'src' => true,
				'height' => true,
				'width' => true,
				'frameborder' => true,
				'allowfullscreen' => true,
			);
		}
	
		return $tags;
	}

	public function cron_schedules($schedules){
		$schedules['rl_one_minute'] = array(
			'interval' => 60,
			'display' => esc_html__( 'Every One Minute' )
		);

		return $schedules;
	}

	public function import_process(){
		$this->running_cron = true;

		$task_manager = new \RL\TaskManager(\RL\Admin\Import::get_import_dir());

		if($task_manager->active()){
			if(!$task_manager->running()){
				$task_manager->log('ALL TASKS COMPLETED');
				return;
			}
		}else{
			return;
		}

		$task_manager->log('Background task started...');

		$start_time = microtime(true);

		$time_limit = 50;

		if((string)ini_get('max_execution_time') == (string)(int)ini_get('max_execution_time')){
			$time_limit = (int)ini_get('max_execution_time') * 0.75;
		}

		$task_manager->log('Safe execution time limit - '.$time_limit.'s');

		$count = 0;
		$limit = 10000;

		while($count < $limit){
			if(!$task_manager->running()){
				break;
			}

			$count++;

			$elapsed_time = microtime(true) - $start_time;

			if($elapsed_time > $time_limit){
				break;
			}

			$todo_file_path = $task_manager->get_next_task();

			if($todo_file_path){
				if(file_exists($todo_file_path)){
					$todo_file_contents = json_decode(file_get_contents($todo_file_path), true);

					$task_manager->log('Task started ('.$todo_file_contents['action'].')');

					switch($todo_file_contents['action']){
						case 'category':
							\RL\Import::import_category($task_manager, $todo_file_contents['data']);
						break;
						case 'attachment':
							\RL\Import::import_attachment($task_manager, $todo_file_contents['data']);
						break;
						case 'product':
							\RL\Import::import_product($task_manager, $todo_file_contents['data']);
						break;
						case 'add_category':
							\RL\Import::add_category($task_manager, $todo_file_contents['data']);
						break;
					}

					if(file_exists($todo_file_path)){
						unlink($todo_file_path);
					}

					$task_manager->log('Task finished ('.$todo_file_contents['action'].')');
				}
			}
		}

		$task_manager->log('Background task finished...');

		if($task_manager->running()){
			self::init();
		}
	}

	public function deactivate(){
		$timestamp = wp_next_scheduled('rl_cron_import_process');
		wp_unschedule_event($timestamp, 'rl_cron_import_process');
	}

}
