<?php
namespace RL;

class Import {

    public static function add_category($task_manager, $data){
        $rapidline = new \Rapidline\API(get_option('rl_api_key'));

        $category_id = $data['category_id'];

        $task_manager->log('Adding new category tasks ('.$category_id.')...');

        $category = $rapidline->category($category_id);

		if($category['parent_id'] !== 'top_parent'){
			self::add_category($task_manager, [
				'category_id' => $category['parent_id']
			]);
		}

        $task_manager->add_todo('attachment', $category['feature_image']);

        $task_manager->add_todo('category', $category);

        $products = $rapidline->products($category_id);

        foreach($products as $product){
            if(isset($product['gallery'])){
                foreach($product['gallery'] as $attachment){
                    $task_manager->add_todo('attachment', $attachment);
                }
            }

            $task_manager->add_todo('product', $product);
        }
    }
    public static function import_category($task_manager, $data){
        if(!$data){
            return true;
        }

        $settings = get_option('rapidline_settings');
    
        if(!$settings){
            $settings = array();
        }
    
        $category_id = rapidline_object('category', $data['id']);
    
        if(!$category_id){
            $parent_id = 0;
    
            if($data['parent_id'] !== 'top_parent'){
                $parent_id = rapidline_object('category', $data['parent_id']);
            }
    
            if($parent_id === false){
                $parent_id = 0;
            }
    
            $category_data = array(
                'cat_name' => $data['name'],
                'category_parent' => $parent_id
            );

            $term = wp_insert_term(
                $data['name'],
                'product_cat',
                array(
                    'parent' => $parent_id
                )
            );

			if(is_wp_error($term)){
				if($term->get_all_error_data('term_exists')[0]){
					$category_id = $term->get_all_error_data('term_exists')[0];

					$task_manager->log('Category already exists ('.$category_id.')...');

					$settings = get_option('rapidline_settings');
			
					if(!$settings){
						$settings = array();
					}
			
					$settings[$data['id']]['category_id'] = $category_id;
			
					update_option('rapidline_settings', $settings);
				}else{
					$task_manager->log('Creating category failed - '.$data['name']);
				}
			}else{
				$category_id = isset( $term['term_id'] ) ? $term['term_id'] : 0;
		
				add_term_meta($category_id, '_rapidline_id', $data['id'], true);
		
				$settings = get_option('rapidline_settings');
		
				if(!$settings){
					$settings = array();
				}
		
				$settings[$data['id']]['category_id'] = $category_id;
		
				update_option('rapidline_settings', $settings);
		
				$attachment_id = rapidline_object('attachment', $data['feature_image']['id']);
		
				if($attachment_id){
					add_term_meta( $category_id, 'thumbnail_id', absint( $attachment_id ) );
				}
			}
        }
    
        return true;
    }
    
    public static function import_attachment($task_manager, $data){
        $post_id = null;
    
        $attachment_id = rapidline_object('attachment', $data['id']);
    
        if(!$attachment_id){
            $tmp_path = tempnam(sys_get_temp_dir(), 'rl_importer');
            
            if(file_download($data['url'], $tmp_path)){
                $attachment_id = wp_attachment_insert_path($tmp_path, $post_id, basename($data['url']));
    
                if($attachment_id){
                    update_post_meta($attachment_id, '_rapidline_id', $data['id']);
                }else{
                    return $data['url'].' - Attachment Not Added To Wordpress!';
                }
            }else{
                return $data['url'].' - Image Download Failed!';
            }
        }
    
        return true;
    }
    
    public static function import_product($task_manager, $data){
		$settings = get_option('rapidline_settings');

		if(!$settings){
			$settings = array();
		}

		$markup = 100;

		$main_category = null;

		foreach($data['categories'] as $main_category){
			if(isset($settings[$main_category['id']])){
				if(!empty($settings[$main_category['id']]['import']) and $settings[$main_category['id']]['import']){
					if(!empty($settings[$main_category['id']]['markup'])){
						$markup = $settings[$main_category['id']]['markup'];

						break;
					}
				}
			}
		}

        $post_id = null;
    
        $product_id = rapidline_object('product', $data['id']);
    
        if(!$product_id){
            if(count($data['variations']) > 1){
            	$product = wc_get_product_object_type('variable');
			}else{
				$product = wc_get_product_object_type('simple');
			}

            if(!$product){
				$task_manager->log('Product creation failed - '.$data['name']);
                return false;
			}

            $product->set_name( $data['name'] );
            $product->set_short_description( $data['short_description'] );
            $product->set_status('publish');
            $product->set_catalog_visibility('visible');
    
            $product_id = $product->save();
    
            update_post_meta($product_id, '_rapidline_id', $data['id']);
        }
    
        $product = wc_get_product($product_id);

        update_post_meta($product_id, '_features', $data['features']);
        update_post_meta($product_id, '_specifications', $data['specifications']);
        update_post_meta($product_id, '_box_dimensions', $data['box_dimensions']);
    
        $product->set_short_description($data['short_description']);
        $product->set_description($data['features']);
    
        $first = true;
    
        $gallery_image_ids = [];
    
        if(isset($data['gallery'])){
            foreach($data['gallery'] as $attachment){
                $attachment_id = rapidline_object('attachment', $attachment['id']);
        
                if($attachment_id){
                    $gallery_image_ids[] = $attachment_id;
        
                    if($first){
                        $first = false;
                        $product->set_image_id($attachment_id);
                    }
                }
            }
        }
    
        $product->set_gallery_image_ids($gallery_image_ids);
    
        $category_ids = [];
    
        foreach($data['categories'] as $category){
            $category_id = rapidline_object('category', $category['id']);
    
            if($category_id){
                $category_ids[] = $category_id;
            }
        }
    
        $product->set_category_ids($category_ids);
    
        // Handle Variations

		$current_children = [];

		if(is_a($product, '\\WC_Product_Variable')){
			$current_children = $product->get_children();
		}

		if(count($data['variations']) > 1){
			$colours_attribute_name = 'pa_'.wc_sanitize_taxonomy_name(wp_unslash(RL_IMPORTER_COLOURS_ATTRIBUTE));
			$colours_attribute_id = wc_attribute_taxonomy_id_by_name(RL_IMPORTER_COLOURS_ATTRIBUTE);

			$attributes_raw = [];
		
			foreach($data['variations'] as $variation){
				foreach($variation['filters'] as $filter_name => $filter_value){
					$attributes_raw[$filter_name][] = $filter_value;
				}
			}
		
			$attributes = [];
		
			foreach($attributes_raw as $attribute_name => $attribute_options){
				if(count($attribute_options) <= 1){
					continue;
				}

				$attribute_object = new \WC_Product_Attribute();
		
				$attribute_object->set_name($attribute_name);
				$attribute_object->set_visible(false);
				$attribute_object->set_variation(true);

				$attribute_object->set_options($attribute_options);
		
				$attributes[sanitize_title($attribute_name)] = $attribute_object;
			}
		
			$product->set_attributes($attributes);
		
			foreach($data['variations'] as $variation){
				$attributes = [];
		
				foreach($variation['filters'] as $filter_name => $filter_value){
					if(count($attributes_raw[$filter_name]) <= 1){
						continue;
					}

					$attributes[sanitize_title($filter_name)] = $filter_value;
				}

				$variation_id = wc_get_product_id_by_sku($variation['sku']);
				$parent_product_id = wp_get_post_parent_id($variation_id);

				if($parent_product_id == $product->get_id()){
					$variation_object = wc_get_product($variation_id);
				}else{
					$variation_object = wc_get_product_object_type('variation');
					$variation_object->set_parent_id($product_id);
				}
		
				$variation_object->set_attributes($attributes);
				$variation_object->set_status('publish');

				$price_markup = $variation['regular_price'] * ($markup/100);

				$variation_object->set_price($price_markup);
				$variation_object->set_regular_price($price_markup);
				
				if(!self::sku_exists($variation['sku'])){
					try{
						$variation_object->set_sku($variation['sku']);
					}catch(\WC_Data_Exception $e){
						$task_manager->log('SKU already exists and so could not be set for new variation - '.$variation['sku']);
					}
				}else{
					$task_manager->log('SKU already exists and so could not be set for new variation - '.$variation['sku']);
				}

				$variation_object->save();

				$current_children_index = array_search($variation_object->get_id(), $current_children);

				if($current_children_index !== false){
					unset($current_children[$current_children_index]);
				}
			}
		}else{
			if(isset($data['variations'][0])){
				$variation = $data['variations'][0];

				$price_markup = $variation['regular_price'] * ($markup/100);

				$product->set_price($price_markup);
				$product->set_regular_price($price_markup);
				
				if(!self::sku_exists($variation['sku'])){
					$product->set_sku($variation['sku']);
				}else{
					$task_manager->log('SKU already exists and so could not be set for new variation - '.$variation['sku']);
				}
			}else{
				$task_manager->log('Product Information Not Available - '.$data['name']);
			}
		}
    
        $product->save();

		foreach($current_children as $child){
			$child_object = wc_get_product($child);
			$child_object->delete();

			$task_manager->log('Delete Unused Child: '.$child);
		}

        return true;
    }

	static function sku_exists($sku){
		$product = wc_get_product_id_by_sku($sku);

		if($product and is_a($product, 'WC_Product')){
			return true;
		}

		return false;
	}
}
