<?php
namespace RL;

class TaskManager {

    public $path = null;

    public function __construct($path){
        $this->path = $path;
    }

    static public function create($path){
        @mkdir($path.'/data');
        touch($path.'/log');
        file_put_contents($path.'/time', time());

        return new self($path);
    }

    public function get_next_task(){
        $files = $this->list_tasks();

        if(count($files)){
            return $files[0];
        }

        return false;
    }

	public function list_tasks(){
		$files = glob($this->path.'/data/*');

        if(count($files)){
            usort($files, function($a, $b){
                $a_number = (int)substr(basename($a, '.json'), 5);
                $b_number = (int)substr(basename($b, '.json'), 5);

                if ($a_number == $b_number) {
                    return 0;
                }

                return ($a_number < $b_number) ? -1 : 1;
            });

            return $files;
        }

        return [];
	}

    public function cancel(){
		if(file_exists($this->path.'/log')){
			unlink($this->path.'/log');
		}

		if(file_exists($this->path.'/time')){
        	unlink($this->path.'/time');
		}

		foreach($this->list_tasks() as $task){
			unlink($task);
		}
	}

    public function add_todo($action, $data){
        $todo_count = $this->tasks_total();

        $todo_file_path = $this->path.'/data/todo_'.$todo_count.'.json';

        file_put_contents($todo_file_path, json_encode([
            'action' => $action,
            'data' => $data
        ], JSON_PRETTY_PRINT));
    }

    public function log($message){
		if(is_array($message) or is_object($message)){
			$message = print_r($message, true);
		}
		
		file_put_contents($this->path.'/log', '['.date('Y-m-d H:i:s').'] '.$message."\n", FILE_APPEND);
	}

    public function running(){
        if($this->active()){
            return (bool)count($this->list_tasks());
        }

        return false;
    }

    public function active(){
        return file_exists($this->path.'/log');
    }

    public function view_log(){
        if(file_exists($this->path.'/log')){
            return trim(file_get_contents($this->path.'/log'));
        }

        return '';
    }

    public function tasks_total(){
        $tasks = $this->list_tasks();

        $last_task = end($tasks);

        return ((int)substr(basename($last_task, '.json'), 5)) + 1;
    }

    public function tasks_completed(){
        $tasks = $this->list_tasks();

        $last_task = end($tasks);

        $total = ((int)substr(basename($last_task, '.json'), 5)) + 1;

        return $total - count($tasks);
    }

    public function start_time(){
        if(file_exists($this->path.'/time')){
            return (int)trim(file_get_contents($this->path.'/time'));
        }

        return 0;
    }

}